"""
  Project Tongji-EasyAPI
  crack.pyi
  Origin Project : https://github.com/seashell11234455/crackids  License GPL v3.0
  Origin Author:  FeO3  https://github.com/seashell11234455
  www.cinea.com.cn
"""

def getCode()->str: ...
