"""
  Keep love for life! 
  ~~~~~~~~~~~~~~~~~~~~~~~~~~

  Project Tongji-EasyAPI
  __version__.py
  Copyright (c) 2022 Cinea Zhan. All rights reserved
  www.cinea.com.cn
"""

__title__ = 'easyTongjiapi'
__description__ = 'Easy Tongji 1-System API for everyone.'
__url__ = 'https://www.cinea.com.cn'
__version__ = '0.1.0'
__build__ = 0x001000
__author__ = 'Cinea Zhan'
__author_email__ = 'cinea@cinea.com.cn'
__license__ = 'GPL v3.0'
__copyright__ = 'Copyright 2022 Cinea Zhan'
__65472__ = 'RRRRRR\nBBBBB\nRRRR\nBBBBBBB\nRR\n分流政策是SB'